#include "config.h"

#include <mysql.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "loaddata.h"

MYSQL	mysql;

/*
 *  Open Database and set Global Database Handle 
 */

int opendb(void ) {
		if (!(mysql_connect(&mysql, DB_HOST, DB_USER, DB_PASSWORD))) {
				fprintf(stderr, "%s\n", mysql_error(&mysql) );
				return(1);
		}

		if (mysql_select_db(&mysql, DB_NAME)) {
				fprintf(stderr, "%s\n", mysql_error(&mysql) );
				return(1);	
		}

		return(0);		
}
/*
 * 
 *  Get destination networks per source netgroup id
 * 
 * 
 * 
 */
int getdstnetworks(struct iparoot_struct *root, int sng) {
	MYSQL_RES			*qresult;
	MYSQL_ROW			row;
	int					nrow,
						i, j,
						ngnum;
	char				querystring[255],
						*dummy;

	/* Insert netgroup no into query */	
	sprintf(querystring, QUERY_DSTNW, sng);

	/* Query source networks with unresolved dst networks */
	if (mysql_query(&mysql, querystring)) {
		fprintf(stderr, "Error in mysql query - %s\n", mysql_error(&mysql));
		return(0);
	}

	/* Store result - otherwise exit unsuccessful */
	if (!(qresult=mysql_store_result(&mysql))) {
		fprintf(stderr, "Error in mysql_store_result - %s\n", mysql_error(&mysql));
		return(0);
	}
	/* Did we fetch a single line ? */	
	if (!(nrow=mysql_num_rows(qresult))) {
		fprintf(stderr, "Did not find a single line in table net2net\n");
		return(0);
	}

	/* Malloc memory for root structure */
	if (!(root->destination=malloc((nrow + 1) * sizeof(struct ipan_struct)))) {
		fprintf(stderr, "Ops - out of memory\n");
		mysql_free_result(qresult);
		return(0);
	}	

	/* Clear memory */
	memset(root->destination, 0, (nrow + 1) * sizeof(struct ipan_struct));

	j=ngnum=-1;
	for(i=0;i<nrow;i++) {
		if (!(row=mysql_fetch_row(qresult))) {
			fprintf(stderr, "Error in mysql_fetch_row - %s\n", mysql_error(&mysql));
			mysql_free_result(qresult);
	    	return(0);
		}

		/* netgroup id */
		root->destination[i].real=atol(row[0]);	

		/* Did the netgroup id change ? */
		if (j != root->destination[i].real) {
				ngnum++;
				j=root->destination[i].real;
		}

		/* Set offset to dst network table */
		root->destination[i].value=ngnum*root->no_address;

		/* network begin & end */
		root->destination[i].begin=strtoul(row[1],&dummy,10);
		root->destination[i].end=strtoul(row[2],&dummy,10);
	}

	ngnum++;

	mysql_free_result(qresult);

	/* Add +1 otherwise last netgroup wont be dumped to db */
	root->no_netgroup=ngnum+1;
	
	return(ngnum);
}

/* Return 0 if ok */
int getsrcnetworks(struct iparoot_struct *root, int sng) {
	MYSQL_RES			*qresult;
	MYSQL_ROW			row;
	int					rows,
						i,
						memsize;
	unsigned long 		ipnum=0;
	char				querystring[255],
						*dummy;
#ifdef DEBUG
	printf("getsrcnetworks: getting netgroup %d\n",sng);
#endif
	/* Insert netgroup no into query */	
	sprintf(querystring, QUERY_SOURCENW, sng);

	/* Query source networks with unresolved dst networks */
	if (mysql_query(&mysql, querystring)) {
		fprintf(stderr, "Error in mysql query - %s\n", mysql_error(&mysql));
		return(0);
	}

	/* Store result - otherwise exit unsuccessful */
	if (!(qresult=mysql_store_result(&mysql)))
		return(0);

	/* Did we fetch a single line ? */	
	if (!(rows=mysql_num_rows(qresult))) {
		fprintf(stderr, "Did not find a single line in table net2net\n");
		return(0);
	}

	/* Calc needed memsize ... */
	memsize=(rows + 1) * sizeof(struct ipan_struct);
	/* Malloc memory for root structure */
	if (!(root->source=malloc(memsize))) {
		fprintf(stderr, "Ops - out of memory\n");
		return(0);
	}	

	/* Clear memory */
	memset(root->source, 0, memsize);

	for(i=0;i<rows;i++) {

		if (!(row=mysql_fetch_row(qresult)))
				return(0);

		root->source[i].begin=strtoul(row[0], &dummy, 10);	
		root->source[i].end=strtoul(row[1], &dummy, 10);	
		root->source[i].value=ipnum;

		ipnum+=root->source[i].end-root->source[i].begin+1;
	}

	/* Set total number of source ip addresses */
	root->no_address=ipnum;

	return(ipnum);

}

struct iparoot_struct *get_array(void ) {
	MYSQL_RES			*qresult;
	MYSQL_ROW			snwrow;
	int					snwrows,
						i,
						sng,
						memsize;
	struct iparoot_struct 	*root;

	/* Query source networks with unresolved dst networks */
	if (mysql_query(&mysql, QUERY_SOURCENG)) {
		fprintf(stderr, "Error in mysql query - %s\n", mysql_error(&mysql));
		return(NULL);
	}

	/* Store result - otherwise exit unsuccessful */
	if (!(qresult=mysql_store_result(&mysql)))
			return(NULL);

	/* Did we fetch a single line ? */	
	if (!(snwrows=mysql_num_rows(qresult))) {
			fprintf(stderr, "Did not find a single line in table net2net\n");
			return(NULL);
	}

#ifdef DEBUG
	printf("Found %d source networks\n", snwrows);
#endif

	/* Malloc memory for root structure */
	if (!(root=malloc((snwrows + 1) * sizeof(struct iparoot_struct)))) {
			fprintf(stderr, "Ops - out of memory\n");
			return(NULL);
	}	

	/* Clear memory */
	memset(root, 0, sizeof((snwrows + 1) * sizeof(struct iparoot_struct)));

	/* Loop on source network rows with unresolved destination networks */
	for(i=0;i<snwrows;i++) {
				/* Fetch row from query result */
				if (!(snwrow=mysql_fetch_row(qresult))) {
					fprintf(stderr, "Ops - less rows in table than expected\n");
					return(NULL);
				}
				/* Source network group */
				sng=atol(snwrow[0]);
				
				if (!(getsrcnetworks(&root[i], sng)))
						return(NULL);

				if (!(getdstnetworks(&root[i], sng)))
						return(NULL);
			
				memsize=(root[i].no_address+1) * (root[i].no_netgroup+1) * sizeof(DATA_TYPE);	

				if (!(root[i].data=malloc(memsize))) {
						fprintf(stderr, "Ops - not enough memory for data array\n");
						return(NULL);
				}

				memset(root[i].data,0,memsize); 
	}
	return(root);
}


struct iparoot_struct *buildarray(void ) {
	struct iparoot_struct *result;

	/* Open DB */
	if (opendb())
			return(0);

	/* Get result and return if successfull */
	if ((result=get_array()))
			return(result);
	
	return(NULL);
}
